/*
Filename: tf_bram32x16.v 
Version: 1.0 

Date: April 2000
                                            
Description: Testfixture that stimulates the top-level design
		 top_bram32x16.v.

---------------------------------------------------------------------
XAPP 410                                 
Simulating a 3.1i Core generator Verilog Design
                                           
Instantiate a Core Generator BRAM configured 
as 32x16 and a bi-directional bus for the DATA in/out.       
Compare Data data files being written out. Observe the 
testbench read in a file and do a check with simulation 
results.
 
Limited Warranty and Disclaimer. These designs are provided to you 
as is. Xilinx and its licensors make and you receive no warranties 
or conditions, express, implied, statutory or otherwise, and Xilinx 
specifically disclaims any implied warranties of merchantability, 
non-infringement, or fitness for a particular purpose. Xilinx does 
not warrant that the functions contained in these designs will meet 
your requirements, or that the operation of these designs will be 
uninterrupted or error free, or that defects in the Designs will be 
corrected. Furthermore, Xilinx does not warrant or make any 
representations regarding use or the results of the use of the 
designs in terms of correctness, accuracy, reliability, or otherwise. 
 
Limitation of Liability. In no event will Xilinx or its licensors be 
liable for any loss of data, lost profits, cost or procurement of 
substitute goods or services, or for any special, incidental, 
consequential, or indirect damages arising from the use or operation 
of the designs or accompanying documentation, however caused and on 
any theory of liability. This limitation will apply even if Xilinx 
has been advised of the possibility of such damage. This limitation 
shall apply not-withstanding the failure of the essential purpose of 
any limited remedies herein. 

---------------------------------------------------------------------
*/


`include "glbl.v"

`timescale 1 ns / 1 ps

module test;
reg [4:0] ADDRA, ADDRB;
reg CLK, WEA, WEB;
wire [15:0] DATA_BUSA, DATA_BUSB;

// Setup Debugging variables
reg [15:0] expectedA, expectedB;
integer error_cntA, error_cntB;

// Shadow register for procedural assignments
reg [15:0] data_regA, data_regB;

// Transfer shadow register to data bus
assign DATA_BUSA = WEA ? data_regA : 16'hzzzz;   
assign DATA_BUSB = WEB ? data_regB : 16'hzzzz;

// /////////////////////////////
// Instantiatation of the Design
// /////////////////////////////

top_bram32x16 UUT (.CLK (CLK), .DATA_BUSA (DATA_BUSA), .ADDRA (ADDRA), 
     .WEA (WEA), .DATA_BUSB (DATA_BUSB), .ADDRB (ADDRB), .WEB (WEB));

// ///////////////////////////////
// Wait till Global Reset Finished
// ///////////////////////////////

reg GSR;
assign glbl.GSR = GSR;

`define GSR_PULSE 100

initial begin
  GSR = 1;
  #`GSR_PULSE GSR = 0;
  $display("GSR released ... %t", $stime);
end
 
// ////////////////
// Create the clock
// ////////////////

`define CLK_PERIOD 60

initial begin 
  CLK = 0;
  #`GSR_PULSE forever #(`CLK_PERIOD / 2) CLK = ~CLK;
end
 
/*
initial begin
  $dumpfile("bram_s16_s16.vcd");  // Create a waveform database
  $dumpvars(0, test);             // Probe all nodes at the highest scope 
end
*/

initial begin
  $timeformat(-9, 1, " ns", 9);  // Display time in ns
end

initial begin

  // //////////////////////////
  // Initialize All Input Ports
  // //////////////////////////

  data_regA = 16'h0000; data_regB = 16'h0000;
  WEA = 0; WEB = 0;
  ADDRA = 5'h00; ADDRB = 5'h1F;

#`GSR_PULSE
  // Read back alternating patterns from mem locations
  // Odd  locations to 0101010101010101 = h'5555
  // Even locations to 1010101010101010 = h'AAAA

  repeat (15) begin
    #`CLK_PERIOD
    // Step though all 32 memory locations
    ADDRA = ADDRA + 1;
    ADDRB = ADDRB - 1;
  end

#120
  $stop;

end

// /////////////////////
// Self-checking process
// /////////////////////

initial begin

  // //////////////////////////
  // Initialize debug variables
  // //////////////////////////

  error_cntA = 0; error_cntB = 0;
  expectedA = 16'hAAAA; expectedB = 16'h5555;

end

initial begin
#(`GSR_PULSE - 1)
  $display("Read back alternating patterns ... ");
  $display("Doing block read from 32 mem addresses ... %t", $stime);

  repeat (16) begin
    #`CLK_PERIOD
        check_dataA;
        check_dataB;

        // Alternate between checking for AAAA and 5555
        expectedA = ~expectedA;
        expectedB = ~expectedB;
    end

  $display("\n Port A: Completed tests with %0d errors!", error_cntA);	
  $display("\n Port B: Completed tests with %0d errors!", error_cntB);	

end

//
// Subroutines to compare the BlockRAM outputs to expected results
//

task check_dataA;
  if (DATA_BUSA !== expectedA)	// Check data
    begin
      $display("ERROR: (Port A) %t - Read: %h from address %h:",
	$realtime, DATA_BUSA, ADDRA, " Expected: %h", expectedA);
      error_cntA = error_cntA + 1;
    end
endtask

task check_dataB;
  if (DATA_BUSB !== expectedB)	// Check data
    begin
      $display("ERROR: (Port B) %t - Read: %h from address %h:",
  	$realtime, DATA_BUSB, ADDRB, " Expected: %h", expectedB);
      error_cntB = error_cntB + 1;
    end
endtask

endmodule
